==> mongodb-src-r3.0.5.tar.gz: mongodb-src-r3.0.5/src/mongo/platform/atomic_intrinsics_gcc_atomic.h <==
/*    Copyright 2012 10gen Inc.
 *
 *    This program is free software: you can redistribute it and/or  modify
 *    it under the terms of the GNU Affero General Public License, version 3,
 *    as published by the Free Software Foundation.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Affero General Public License for more details.
 *
 *    You should have received a copy of the GNU Affero General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    As a special exception, the copyright holders give permission to link the
 *    code of portions of this program with the OpenSSL library under certain
 *    conditions as described in each individual source file and distribute
 *    linked combinations including the program with the OpenSSL library. You
 *    must comply with the GNU Affero General Public License in all respects
 *    for all of the code used other than as permitted herein. If you modify
 *    file(s) with this exception, you may extend this exception to your
 *    version of the file(s), but you are not obligated to do so. If you do not
 *    wish to do so, delete this exception statement from your version. If you
 *    delete this exception statement from all source files in the program,
 *    then also delete it in the license file.
 */

/**
 * Implementation of the AtomicIntrinsics<T>::* operations for systems on any
 * architecture using a new enough GCC-compatible compiler toolchain.
 */

#pragma once

namespace mongo {

    /**
     * Instantiation of AtomicIntrinsics<> for all word types T.
     */
    template <typename T, typename IsTLarge=void>
    class AtomicIntrinsics {
    public:

        static T compareAndSwap(volatile T* dest, T expected, T newValue) {
            __atomic_compare_exchange(dest, &expected, &newValue, false, __ATOMIC_SEQ_CST, __ATOMIC_SEQ_CST);
            return expected;
        }

        static T swap(volatile T* dest, T newValue) {
            T result;
            __atomic_exchange(dest, &newValue, &result, __ATOMIC_SEQ_CST);
            return result;
        }

        static T load(volatile const T* value) {
            T result;
            __atomic_load(value, &result, __ATOMIC_SEQ_CST);
            return result;
        }

        static T loadRelaxed(volatile const T* value) {
            T result;
            __atomic_load(value, &result, __ATOMIC_RELAXED);
            return result;
        }

        static void store(volatile T* dest, T newValue) {
            __atomic_store(dest, &newValue, __ATOMIC_SEQ_CST);
        }

        static T fetchAndAdd(volatile T* dest, T increment) {
            return __atomic_fetch_add(dest, increment, __ATOMIC_SEQ_CST);
        }

    private:
        AtomicIntrinsics();
        ~AtomicIntrinsics();
    };

}  // namespace mongo

==> mongodb-src-r3.0.14.tar.gz: mongodb-src-r3.0.14/src/mongo/platform/atomic_intrinsics_gcc_atomic.h <==
/*    Copyright 2012 10gen Inc.
 *
 *    This program is free software: you can redistribute it and/or  modify
 *    it under the terms of the GNU Affero General Public License, version 3,
 *    as published by the Free Software Foundation.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Affero General Public License for more details.
 *
 *    You should have received a copy of the GNU Affero General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    As a special exception, the copyright holders give permission to link the
 *    code of portions of this program with the OpenSSL library under certain
 *    conditions as described in each individual source file and distribute
 *    linked combinations including the program with the OpenSSL library. You
 *    must comply with the GNU Affero General Public License in all respects
 *    for all of the code used other than as permitted herein. If you modify
 *    file(s) with this exception, you may extend this exception to your
 *    version of the file(s), but you are not obligated to do so. If you do not
 *    wish to do so, delete this exception statement from your version. If you
 *    delete this exception statement from all source files in the program,
 *    then also delete it in the license file.
 */

/**
 * Implementation of the AtomicIntrinsics<T>::* operations for systems on any
 * architecture using a new enough GCC-compatible compiler toolchain.
 */

#pragma once

namespace mongo {

/**
 * Instantiation of AtomicIntrinsics<> for all word types T.
 */
template <typename T, typename IsTLarge = void>
class AtomicIntrinsics {
public:
    static T compareAndSwap(volatile T* dest, T expected, T newValue) {
        __atomic_compare_exchange(
            dest, &expected, &newValue, false, __ATOMIC_SEQ_CST, __ATOMIC_SEQ_CST);
        return expected;
    }

    static T swap(volatile T* dest, T newValue) {
        T result;
        __atomic_exchange(dest, &newValue, &result, __ATOMIC_SEQ_CST);
        return result;
    }

    static T load(volatile const T* value) {
        T result;
        __atomic_load(value, &result, __ATOMIC_SEQ_CST);
        return result;
    }

    static T loadRelaxed(volatile const T* value) {
        T result;
        __atomic_load(value, &result, __ATOMIC_RELAXED);
        return result;
    }

    static void store(volatile T* dest, T newValue) {
        __atomic_store(dest, &newValue, __ATOMIC_SEQ_CST);
    }

    static T fetchAndAdd(volatile T* dest, T increment) {
        return __atomic_fetch_add(dest, increment, __ATOMIC_SEQ_CST);
    }

private:
    AtomicIntrinsics();
    ~AtomicIntrinsics();
};

}  // namespace mongo
